#include <algorithm>
#include <array>
#include <bitset>
#include <cstdint>
#include <iostream>
#include <string>
#include <string_view>
#include <vector>

constexpr char OPS[] = "^|&=!(";
constexpr int8_t FULL = 0xf;

int32_t getPrecedence(char op) {
    return std::find(OPS, OPS + sizeof(OPS), op) - OPS;
}

constexpr bool isUnary(char op) {
    return op == '!';
}

constexpr bool isRightAssociative(char op) {
    return op == '!';
}

constexpr int8_t doOp(char op, int8_t x, int8_t y) {
    if (op == '=') {
        return ~(x ^ y) & FULL;
    } else if (op == '&') {
        return x & y;
    } else if (op == '|') {
        return x | y;
    } else if (op == '^') {
        return x ^ y;
    }

    return 0;
}

// Shunting yard algorithm.
int8_t parse(std::string_view program) {
    std::vector<int8_t> vals;
    std::vector<char> ops;
    vals.reserve(program.size());
    ops.reserve(program.size());

    auto useOp = [&]() {
        char op = ops.back();
        ops.pop_back();

        if (isUnary(op)) {
            vals.back() = ~vals.back() & FULL;
        } else {
            auto y = vals.back();
            vals.pop_back();
            auto x = vals.back();
            vals.pop_back();

            vals.push_back(doOp(op, x, y));
        }
    };

    for (char c : program) {
        if (c == '(') {
            ops.push_back(c);
        } else if (c == ')') {
            while (!ops.empty() && ops.back() != '(') {
                useOp();
            }

            ops.pop_back();
        } else if (c == 'x') {
            vals.push_back(0b1100);
        } else if (c == 'y') {
            vals.push_back(0b1010);
        } else if (c == '0') {
            vals.push_back(0b0000);
        } else if (c == '1') {
            vals.push_back(0b1111);
        } else {
            auto prec = getPrecedence(c);
            while (!ops.empty() && ops.back() != '(' && (getPrecedence(ops.back()) >= prec) && !(c == ops.back() && isRightAssociative(c))) {
                useOp();
            }
            ops.push_back(c);
        }
    }

    while (!ops.empty()) {
        useOp();
    }

    return vals[0];
}

struct Statement {
    int8_t lhs;
    int8_t rhs;
    int8_t expr;
    bool res;
};

bool run(int8_t expr, bool x, bool y) {
    return expr >> x >> x >> y & 1;
}

std::vector<Statement> findUseful(int32_t k, const std::vector<Statement>& code) {
    std::vector<Statement> useful;
    useful.reserve(4 * k * k);
    std::vector<int32_t> valid(k * k, 0xf);
    std::array<int32_t, 4> uses{};
    for (const auto& cur : code) {
        auto [a, b, expr, r] = cur;
        int32_t idx = k * a + b;
        int32_t numUses = 0;
        for (auto xy = 0; xy < 4; ++xy) {
            if ((valid[idx] >> xy & 1) && run(expr, xy >> 1, xy & 1)) {
                uses[numUses++] = xy;
            }
        }

        if (numUses > 0) {
            useful.push_back(cur);

            for (int32_t i = 0; i < numUses; ++i) {
                valid[idx] ^= (1 << uses[i]);
            }
        }
    }

    return useful;
}

bool execute(const std::vector<Statement>& code, int32_t x, int32_t y, bool def) {
    for (const auto& [a, b, expr, r] : code) {
        if (run(expr, x >> a & 1, y >> b & 1)) {
            return r;
        }
    }

    return def;
}

int main() {
    std::ios_base::sync_with_stdio(false);
    std::cin.tie(nullptr);

    int32_t n, k;
    std::cin >> n >> k;

    std::vector<Statement> code;
    code.reserve(n);
    {
        std::string expr;
        for (int32_t i = 0; i < n; ++i) {
            int32_t a, b, r;
            std::cin >> a >> b >> expr >> r;

            a = k - a;
            b = k - b;
            code.push_back({.lhs = static_cast<int8_t>(a), .rhs = static_cast<int8_t>(b), .expr = parse(expr), .res = r > 0});
        }
    }

    int32_t defInt;
    std::cin >> defInt;

    bool def = defInt;

    code = findUseful(k, code);

    std::vector<std::bitset<1024>> res(1 << k);

    for (int32_t a = 0; a < (1 << k); ++a) {
        for (int32_t b = 0; b < (1 << k); ++b) {
            res[a][b] = execute(code, a, b, def);
        }
    }

    int32_t reflexive = 0;
    for (int32_t a = 0; a < (1 << k); ++a) {
        reflexive += res[a][a];
    }

    int32_t symmetric = 0;
    for (int32_t a = 0; a < (1 << k); ++a) {
        for (int32_t b = 0; b < (1 << k); ++b) {
            symmetric += res[a][b] && res[b][a];
        }
    }

    int32_t transitive = 0;
    for (int32_t a = 0; a < (1 << k); ++a) {
        for (int32_t b = 0; b < (1 << k); ++b) {
            if (res[a][b]) {
                auto c = ~res[a] & res[b];

                transitive += c.count();
            }
        }
    }

    std::cout << reflexive << ' ' << symmetric << ' ' << transitive << '\n';

    return 0;
}
