import java.util.ArrayDeque
import java.util.BitSet

val FULL = 0b1111

fun getPrecedence(op: Char): Int {
    return "^|&=!(".indexOf(op)
}

fun doOp(op: Char, x: Int, y: Int): Int {
    if (op == '=') {
        return (x xor y).inv() and FULL
    } else if (op == '&') {
        return x and y
    } else if (op == '|') {
        return x or y
    } else if (op == '^') {
        return x xor y
    }

    return 0
}

fun parse(program: String): Int {
    var vals = ArrayDeque<Int>()
    var ops = ArrayDeque<Char>()

    fun useOp() {
        val op = ops.pop()
        if (op == '!') {
            val x = vals.pop()
            vals.push(x.inv() and FULL)
        } else {
            val y = vals.pop()
            val x = vals.pop()

            vals.push(doOp(op, x, y))
        }
    }

    for (c in program) {
        if (c == '(') {
            ops.push(c)
        } else if (c == ')') {
            while (!ops.isEmpty() && ops.peek() != '(') {
                useOp()
            }

            ops.pop()
        } else if (c == 'x') {
            vals.push(0b1100)
        } else if (c == 'y') {
            vals.push(0b1010)
        } else if (c == '0') {
            vals.push(0b0000)
        } else if (c == '1') {
            vals.push(FULL)
        } else {
            val prec = getPrecedence(c)
            while (!ops.isEmpty() && ops.peek() != '(' && getPrecedence(ops.peek()) >= prec && (c == ops.peek() && c == '!').not()) {
                useOp()
            }
            ops.push(c)
        }
    }

    while (!ops.isEmpty()) {
        useOp()
    }

    return vals.getFirst()
}

fun run(expr: Int, x: Int, y: Int): Boolean {
    return (expr shr (2 * x + y) and 1) > 0
}

typealias Statement = IntArray

fun findUseful(k: Int, code: List<Statement>): List<Statement> {
    val useful = mutableListOf<Statement>()
    val valid = IntArray(k * k) { 0xf }
    val uses = IntArray(4) { 0 }
    for (cur in code) {
        val (a, b, expr, _) = cur
        val idx = k * a + b
        var numUses = 0
        for (xy in 0 until 4) {
            if (((valid[idx] shr xy and 1) > 0) && run(expr, xy shr 1, xy and 1)) {
                uses[numUses] = xy
                numUses += 1
            }
        }

        if (numUses > 0) {
            useful.add(cur)
            for (i in 0 until numUses) {
                valid[idx] = valid[idx] xor (1 shl uses[i])
            }
        }
    }

    return useful
}

fun execute(code: List<Statement>, x: Int, y: Int, def: Boolean): Boolean {
    for ((a, b, expr, r) in code) {
        if (run(expr, x shr a and 1, y shr b and 1)) {
            return r > 0
        }
    }

    return def
}

fun main() {
    val (n, k) = readln().split(" ").map { it.toInt() }
    var code = List<Statement>(n) {
        var (a, b, expr, r) = readln().split(" ")

        intArrayOf(k - a.toInt(), k - b.toInt(), parse(expr), r.toInt())
    }

    val def = readln().toInt() > 0

    code = findUseful(k, code)

    val maxMask = (1 shl k)

    val res = Array<BitSet>(maxMask) { BitSet(1 shl k) }

    for (a in 0 until maxMask) {
        for (b in 0 until maxMask) {
            res[a].set(b, execute(code, a, b, def))
        }
    }

    var reflexive = 0
    var symmetric = 0
    var transitive = 0

    for (a in (0 until maxMask)) {
        if (res[a].get(a)) {
            reflexive += 1
        }
    }

    for (a in (0 until maxMask)) {
        for (b in (0 until maxMask)) {
            if (res[a].get(b) and res[b].get(a)) {
                symmetric += 1
            }
        }
    }

    for (a in (0 until maxMask)) {
        for (b in (0 until maxMask)) {
            if (res[a].get(b)) {
                var c = res[a].clone() as BitSet
                c.flip(0, maxMask)
                c.and(res[b])
                var start = c.nextSetBit(0)
                while (start != -1) {
                    transitive += 1
                    start = c.nextSetBit(start + 1)
                }
            }
        }
    }

    println("$reflexive $symmetric $transitive")
}
